#include <avr/io.h>
#include <avr/interrupt.h>
#include "lcd.h"
#include "keyb.h"
#include "ff.h"
#include "diskio.h"

void SDTimerInit(void) {
	OCR0 = F_CPU / 1024 / 100 - 1;
	TCCR0 = _BV(WGM01) | _BV(CS02) | _BV(CS00); //CTC, F_CPU/1024
	TIMSK = _BV(OCIE0); //output compare
}

ISR(TIMER0_COMP_vect) {
	disk_timerproc();
}

void dirList();
void showFile(char * fileName);

int main(void) {
	lcdInit();
	lcdInitPrintf();
	keybInit();
	SDTimerInit(); //timer do komunikacji z kart
	sei();
	FATFS fatFs;
	PORTB |= _BV(PORTB3); // CD pull-up
	lcdWriteCommand(LCD_COMMAND_CLEAR);
	printf("Brak karty");
	while (PINB & _BV(PINB3));
	while(1) {
		FRESULT fresult = f_mount(&fatFs, "", 1);
		if(fresult == FR_OK) {
			lcdWriteCommand(LCD_COMMAND_CLEAR);
			dirList();
		} else {
			lcdWriteCommand(LCD_COMMAND_CLEAR);
			printf("Blad karty");
		}
	}
}

uint8_t getDirItem(DIR * dp, FILINFO * fno, uint8_t itemNumber) {
	f_readdir(dp, NULL);
	while (itemNumber--) f_readdir(dp, fno);
	return fno->fname[0] ? 1 : 0;
}

void showDirItem(FILINFO * fno) {
	//sprawd czy to katalog
	if (fno->fattrib & AM_DIR) {
		printf("#");
	} else {
		printf(" ");
	}
	//wywietl nazw pliku
	printf(fno->fname);
}

void dirList() {
	DIR dp;
	FILINFO fno;
	uint8_t result = 0;
	f_chdir("/");
	f_opendir(&dp, ".");
	uint16_t itemNumber = 1;
	while (1) {
		lcdWriteCommand(LCD_COMMAND_CLEAR);
		printf(">");
		result = getDirItem(&dp, &fno, itemNumber);
		//jeli w katalogu istnieje element o takim numerze
		if (result == 1) {
			//wywietl na LCD
			showDirItem(&fno);
			//wywietl kolejny
			lcdGotoXY(0, 1);
			printf(" ");
			result = getDirItem(&dp, &fno, itemNumber + 1);
			if (result == 1) showDirItem(&fno);
		}
		uint8_t key = getKey();
		if (result == 1) {
			if (key == 5) itemNumber++;
		}
		if (key == 1) itemNumber--;
		if (itemNumber == 0) itemNumber = 1;
		if (key == 15) {
			//wyjd z katalogu
			f_closedir(&dp);
			f_chdir("..");
			f_opendir(&dp, ".");
			itemNumber = 1;
		}
		if (key == 16) {
			//wejd do katalogu
			getDirItem(&dp, &fno, itemNumber);
			if (fno.fattrib & AM_DIR) {
				f_closedir(&dp);
				f_chdir(fno.fname);
				f_opendir(&dp, ".");
				itemNumber = 1;
			} else {
				showFile(fno.fname);
				getKey();
			}
		}
	}
}

void showFile(char * fileName) {
	FIL fp;
	FRESULT fresult = f_open(&fp, fileName, FA_READ);
	if(fresult == FR_OK) {
		char buff[16];
		f_gets(buff, 16, &fp);
		lcdWriteCommand(LCD_COMMAND_CLEAR);
		printf(buff);
		f_close(&fp);
	}
}